from .auth import auth_interface
import pili.conf as conf
from urllib2 import Request
import json

def normalize(args, keyword):
    if set(args) - set(keyword):
        raise ValueError('invalid key')
    for k, v in args.items():
        if v is None:
            del args[k]
    return args


@auth_interface
def create_stream(**args):
    keyword = ['hub', 'title', 'publishKey', 'publishSecurity']
    encoded = json.dumps(normalize(args, keyword))
    url = "http://%s/%s/streams" % (conf.API_HOST, conf.API_VERSION)
    return Request(url=url, data=encoded)

@auth_interface
def get_stream(stream_id):
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url)

@auth_interface
def get_stream_list(**args):
    keyword = ['hub', 'marker', 'limit', 'title', 'status', 'idonly']
    args = normalize(args, keyword)
    if args.has_key('idonly'):
        if args['idonly'] is not True:
            del args['idonly']
    url = "http://%s/%s/streams?" % (conf.API_HOST, conf.API_VERSION)
    for k, v in args.items():
        url += "&%s=%s" % (k, v)
    req = Request(url=url)
    return req

@auth_interface
def update_stream(stream_id, **args):
    keyword = ['publishKey', 'publishSecurity', 'disabled']
    encoded = json.dumps(normalize(args, keyword))
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url, data=encoded)

@auth_interface
def delete_stream(stream_id):
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    req = Request(url=url)
    req.get_method = lambda: 'DELETE'
    return req

@auth_interface
def get_status(stream_id):
    url = "http://%s/%s/streams/%s/status" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url)

@auth_interface
def get_segments(stream_id, start_second=None, end_second=None, limit=None):
    url = "http://%s/%s/streams/%s/segments" % (conf.API_HOST, conf.API_VERSION, stream_id)
    if start_second and end_second:
        url += "?start=%s&end=%s" % (start_second, end_second)
    if limit != None:
        url += "&limit=%s" % (limit)
    return Request(url=url)

@auth_interface
def save_stream_as(stream_id, **args):
    keyword = ['name', 'notifyUrl', 'start', 'end', 'format']
    encoded = json.dumps(normalize(args, keyword))
    url = "http://%s/%s/streams/%s/saveas" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url, data=encoded)

@auth_interface
def snapshot_stream(stream_id, **args):
    keyword = ['name', 'format', 'time', 'notifyUrl']
    encoded = json.dumps(normalize(args, keyword))
    url = "http://%s/%s/streams/%s/snapshot" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url, data=encoded)



import pili.api as api
import pili.conf as conf
from .utils import __hmac_sha1__
import json
import time

class Stream(object):
    """
    Stream is used to control a stream. You should always create a Stream object by the
    interfaces of an application.
    """
    def __init__(self, auth, stream_id=None, data=None):
        self.__auth__ = auth
        if not (stream_id or data):
            raise ValueError('invalid key')
        if not stream_id:
            stream_id = data["id"]
        self.id = stream_id
        self.__data__ = data

    def __getattr__(self, attr):
        if not self.__data__:
            self.refresh()
        try:
            return self.__data__ if attr == "data" else self.__data__[attr]
        except:
            return None

    def refresh(self):
        self.__data__ = api.get_stream(self.__auth__, stream_id=self.id)
        return self

    def update(self, **args):
        res = api.update_stream(self.__auth__, stream_id=self.id, **args)
        self.__data__ = res
        return self

    def disable(self):
        return self.update(disabled=True)

    def enable(self):
        return self.update(disabled=False)

    def status(self):
        res = api.get_status(self.__auth__, stream_id=self.id)
        return res

    def segments(self, start_second=None, end_second=None, limit=None):
        res = api.get_segments(self.__auth__, stream_id=self.id, start_second=start_second, end_second=end_second, limit=limit)
        return res

    def save_as(self, **args):
        res = api.save_stream_as(self.__auth__, stream_id=self.id, **args)
        return res

    def snapshot(self, **args):
        res = api.snapshot_stream(self.__auth__, stream_id=self.id, **args)
        return res

    def delete(self):
        res = api.delete_stream(self.__auth__, stream_id=self.id)
        return res

    def __base__(self, protocol, host, profile):
        url = "%s://%s/%s/%s" % (protocol, host, self.hub, self.title)
        if profile!="":
            url += "@%s" % profile
        return url

    def rtmp_live_urls(self):
        res = dict()
        res["ORIGIN"] = self.__base__("rtmp", self.hosts["live"]["rtmp"], "")
        if self.profiles!=None:
            for profile in self.profiles:
                res[profile] = self.__base__("rtmp", self.hosts["live"]["rtmp"], profile)
        return res

    def http_flv_live_urls(self):
        res = dict()
        res["ORIGIN"] = self.__base__("http", self.hosts["live"]["hdl"], "") + ".flv"
        if self.profiles!=None:
            for profile in self.profiles:
                res[profile] = self.__base__("http", self.hosts["live"]["hdl"], profile) + ".flv"
        return res

    def hls_live_urls(self):
        res = dict()
        res["ORIGIN"] = self.__base__("http", self.hosts["live"]["hls"], "") + ".m3u8"
        if self.profiles!=None:
            for profile in self.profiles:
                res[profile] = self.__base__("http", self.hosts["live"]["hls"], profile) + ".m3u8"
        return res

    def hls_playback_urls(self, start_second, end_second):
        name = str(int(time.time()))
        resp = self.save_as(name=name, start=start_second, end=end_second)
        res = dict()
        res["ORIGIN"] = resp["url"]
        return res

    def rtmp_publish_url(self):
        url = "rtmp://%s/%s/%s" % (self.hosts["publish"]["rtmp"], self.hub, self.title)
        if self.publishSecurity == "static":
            url += "?key=%s" % self.publishKey
        elif self.publishSecurity == "dynamic":
            nonce = str(int(time.time()))
            url += "?nonce=%s" % nonce
            data = "/%s/%s?nonce=%s" % (self.hub, self.title, nonce)
            token = __hmac_sha1__(data, str(self.publishKey))
            url += "&token=%s" % token
        else:
            return None
        return url

    def to_json(self):
        return json.dumps(self.data)



import pili.api as api
from .stream import Stream

class Hub(object):
    def __init__(self, credentials, hub_name):
        self.__auth__ = credentials.__auth__
        self.__hub__ = hub_name

    def create_stream(self, **args):
        res = api.create_stream(self.__auth__, hub=self.__hub__, **args)
        return Stream(self.__auth__, data=res)

    def get_stream(self, stream_id):
        return Stream(self.__auth__, stream_id=stream_id)

    def list_streams(self, **args):
        res = api.get_stream_list(self.__auth__, hub=self.__hub__, **args)
        items = []
        for data in res["items"] if res["items"] is not None else []:
            items.append(Stream(self.__auth__, stream_id=data["id"]))
        res["items"] = items
        return res


"""
Utils
"""
from urllib2 import urlopen, HTTPError
import contextlib, json
from .errors import APIError
import hmac, hashlib, base64

def send_and_decode(req):
    """
    Send the request and return the decoded json of response.

    Args:
        req: urllib2.Request

    Returns:
        A dict of decoded response
    """
    try:
        with contextlib.closing(urlopen(req)) as res:
            if res.getcode() == 204:
                return None
            raw = res.read()
            return json.loads(raw)
    except HTTPError, res:
        raw = res.read()
        try:
            data = json.loads(raw)
        except ValueError:
            raise APIError(res.reason)
        else:
            raise APIError(data["error"])

def __hmac_sha1__(data, key):
    """
    hmac-sha1
    """
    hashed = hmac.new(key, data, hashlib.sha1)
    return base64.urlsafe_b64encode(hashed.digest())



class APIError(RuntimeError):
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return "%s" % (self.message)
    def __repr__(self):
        return self.__str__()

